@@ -3,7 +3,7 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2005 James Aylett
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2015 Olly Betts
  * Copyright 2009 Frank J Bruzzaniti
  * Copyright 2012 Mihai Bivol
  *
@@ -26,7 +26,6 @@
 #include <config.h>
 
 #include <algorithm>
-#include <fstream>
 #include <iostream>
 #include <limits>
 #include <string>
@@ -113,7 +112,7 @@ static map<string, string> commands;
 inline static bool
 p_notalnum(unsigned int c)
 {
-    return !isalnum(static_cast<unsigned char>(c));
+    return !C_isalnum(c);
 }
 
 
@@ -298,7 +297,7 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 	string::iterator i;
 	for (i = ext.begin(); i != ext.end(); ++i) {
 	    if (*i >= 'A' && *i <= 'Z') {
-		*i = tolower(*i);
+		*i = C_tolower(*i);
 		changed = true;
 	    }
 	}
@@ -403,6 +402,15 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 		skip(file, "required filter not installed", SKIP_VERBOSE_ONLY);
 		return;
 	    }
+	    if (cmd == "false") {
+		// Allow setting 'false' as a filter to mean that a MIME type
+		// should be quietly ignored.
+		string m = "ignoring MIME type '";
+		m += cmd_it->first;
+		m += "'";
+		skip(file, m, SKIP_VERBOSE_ONLY);
+		return;
+	    }
 	    append_filename_argument(cmd, file);
 	    try {
 		dump = stdout_to_string(cmd);
@@ -1226,6 +1234,8 @@ main(int argc, char **argv)
     mime_map["js"] = "ignore";
     mime_map["lib"] = "ignore";
     mime_map["lnk"] = "ignore";
+    mime_map["msi"] = "ignore";
+    mime_map["msp"] = "ignore";
     mime_map["o"] = "ignore";
     mime_map["obj"] = "ignore";
     mime_map["pyc"] = "ignore";
@@ -1266,8 +1276,8 @@ main(int argc, char **argv)
 					 longopts, NULL)) != -1) {
 	switch (getopt_ret) {
 	case 'h': {
-	    cout << PROG_NAME" - "PROG_DESC"\n\n"
-"Usage: "PROG_NAME" [OPTIONS] --db DATABASE [BASEDIR] DIRECTORY\n"
+	    cout << PROG_NAME " - " PROG_DESC "\n\n"
+"Usage: " PROG_NAME " [OPTIONS] --db DATABASE [BASEDIR] DIRECTORY\n"
 "\n"
 "DIRECTORY is the directory to start indexing from.\n"
 "\n"
@@ -1354,7 +1364,7 @@ main(int argc, char **argv)
 		}
 	    } else {
 		cerr << "Invalid MIME mapping '" << optarg << "'\n"
-			"Should be of the form ext:type, eg txt:text/plain\n"
+			"Should be of the form ext:type, e.g. txt:text/plain\n"
 			"(or txt: to delete a default mapping)" << endl;
 		return 1;
 	    }
